using System;
using System.Net;
using System.Net.Sockets;
using System.Text;
using System.IO;
using System.Threading;

class Server
{
	static private int ClientNumber = 0;

	private event EventHandler NewTurnEvent;
	private object locker = new object();
	private Socket server = null;
	private int NUMBER;
	
	public Server(IPEndPoint endPoint)
		: this()
	{
		EndPoint = endPoint;
	}
	
	public Server()
	{
		MaxClient = 5;
		NUMBER = (new Random()).Next(1000);
	}
	
	public int MaxClient { get; set; }
	public IPEndPoint EndPoint { get; set; }
	
	public void Listen(IPEndPoint endPoint)
	{
		EndPoint = endPoint;
		Listen();
	}
	
	public void Listen()
	{
		if(EndPoint == null)
		{
			throw new Exception("IPEndPoint missing");
		}
		
		server = new Socket(AddressFamily.InterNetwork, 
							SocketType.Stream, 
							ProtocolType.Tcp);
		server.Bind(EndPoint);
		server.Listen(5);
		
		ThreadPool.SetMaxThreads(MaxClient, 0);
		NewTurnEvent += NewTurnHandler;
		
		Console.WriteLine("A szerver elindult, a szm: {0}", NUMBER);
		
		Socket client = null;
		while(true)
		{
			client = server.Accept();
				
			Console.WriteLine("Kliens bejelentkezett, a kvetkez IP cmrl: {0}",
				((IPEndPoint)client.RemoteEndPoint).Address.ToString());
					
			ThreadPool.QueueUserWorkItem(ClientHandler, client);
		}
	}
	
	private void NewTurnHandler(object source, EventArgs e)
	{
		NUMBER = (new Random()).Next(1000);
		Console.WriteLine("Az j szm: {0}", NUMBER);
	}
	
	private void ClientHandler(object socket)
	{
		Socket client = null;
		string name = String.Empty;
		int result = 0;
		
		try
		{
			client = socket as Socket;
		
			if(client == null)
			{
				throw new ArgumentException();
			}
		
			++ClientNumber;
		
			byte[] data = new byte[7];
			int length = client.Receive(data);
			name = Encoding.ASCII.GetString(data, 0, length);
			
			Console.WriteLine("j jtkos: {0}", name);
			
			int i = 0;
			bool win = false;
			while(i < 5 && win == false)
			{
				data = new byte[128];
				length = client.Receive(data);
				
				GuessNumber(name, 
					int.Parse(Encoding.ASCII.GetString(data, 0, length)), 
					out result);
				
				data = Encoding.ASCII.GetBytes(result.ToString());
				client.Send(data, data.Length, SocketFlags.None);
				
				if(result == 0) { win = true; }
				
				++i;
			}
		}
		catch(Exception e)
		{
			Console.WriteLine(e.Message);
		}
		finally
		{
			client.Close();
			
			if(--ClientNumber == 0 && result == 0)
			{
				NewTurnEvent(this, null);
			}
		}
	}
	
	private void GuessNumber(string name, int number, out int result)
	{
		lock(locker)
		{
			if(NUMBER != -1)
			{
				Console.WriteLine("{0} szerint a szm: {1}", name, number);
			
				if(NUMBER == number)
				{
					Console.WriteLine("{0} kitallta a szmot!", name);
					result = 0;
					NUMBER = -1;
				}
				else if(NUMBER < number)
				{
					result = 1;
				}
				else
				{
					result = -1;
				}
			}
			else result = 2;
		}
		
		Thread.Sleep(300);
	}
}

class Program
{
	static public void Main(string[] args)
	{
		IPEndPoint endPoint = new IPEndPoint(
			IPAddress.Parse(args[0]), 
			int.Parse(args[1]));
		
		Server server = new Server(endPoint);
		server.Listen();
	}
}